# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations
from django.utils.text import slugify


def generate_slug(instance, **query_filters):
    base_slug = slugify(instance.label or ('%s-event' % instance.agenda.label))
    slug = base_slug
    i = 1
    while True:
        queryset = instance._meta.model.objects.filter(slug=slug, **query_filters).exclude(pk=instance.pk)
        if not queryset.exists():
            break
        slug = '%s-%s' % (base_slug, i)
        i += 1
    return slug


def set_slug_on_events(apps, schema_editor):
    Event = apps.get_model('agendas', 'Event')
    for event in Event.objects.all().order_by('-pk'):
        if event.slug and not Event.objects.filter(slug=event.slug).exclude(pk=event.pk).exists():
            continue
        event.slug = generate_slug(event)
        event.save(update_fields=['slug'])


class Migration(migrations.Migration):

    dependencies = [
        ('agendas', '0048_meeting_type_deleted_flag'),
    ]

    operations = [
        migrations.RunPython(set_slug_on_events, lambda x, y: None),
    ]
